#!/usr/bin/env perl

# Run "slides.cgi create=static" to create static html pages.

use strict;
use warnings;
use CGI;

my $q = new CGI;
my $isStatic = $q->param("static")? 1: 0;

my $bgcolor = `grep background ./style.css | head -1 | cut -d: -f2 | cut -d';' -f1`;
$bgcolor =~ s/\s//sg; $bgcolor ||= "#d0d0d0";
$bgcolor = sprintf("#%02x%02x%02x", $1, $2, $3) if $bgcolor =~ /rgb\((\d+),\s*(\d+),\s*(\d+)\)/;

print <<HEADERS unless -t *STDIN || $q->param('create');
Content-Type: text/html

HEADERS

my $indexFile = "proto-index.html";
my @slideIds = grep !/^index$/, map { chomp; $_ }
	`ls proto-*.html | cut -d- -f2 | cut -d. -f1`;

sub idToHiLo ($) {
	my $id = shift;
	return (-1, -1) if !defined $id;
	$id =~ /^(.+)(.)$/ || die "Incorrect id ($id) given\n";
	return ($1, $2);
}

sub idToNum ($) {
	my $id = shift;
	return join('.', idToHiLo($id));
}

sub idToProto ($) {
	my $id = shift;
	return "proto-$id.html";
}

sub idToSlide ($) {
	my $id = shift;
	return $isStatic? "slide-$id.html": "slides.cgi" . ($id eq 'index'? "": "?i=$id");
}

sub extractTitle ($;$) {
	my $file = shift;
	my $bodyHtml = shift || `cat '$file'`;
	my $title = $bodyHtml =~ m{<!-- title: (.*?) -->}? $1:
		$bodyHtml =~ m{<h1>(.*?)</h1>}? $1: die "No title in $file\n";
	return $title;
}


die "No $indexFile found\n" unless -r $indexFile;
my $indexHtml = `cat '$indexFile'`;
my $mainTitle = extractTitle($indexFile, $indexHtml);


if (defined $q->param("create")) {
	print "Creating the slide index ... ";
	$isStatic = $q->param("create") =~ /^s/i? 1: 0;
	foreach my $id ('index', ($isStatic? @slideIds: ())) {
		my @args = ($0);
		push @args, "autostamp=1";
		push @args, "static=1" if $isStatic;
		my $slide = idToSlide($id);
		if ($id eq 'index') {
			push @args, "| tee $slide" if $isStatic;
			$slide = "index.html";
		} else {
			push @args, "i=$id" if $id ne 'index';
		}
		
		system(join(' ', @args) . " >$slide");
	}
	print "done\n";
	exit;
}

if (!defined $q->param("i")) {
	die "No slides found\n" unless @slideIds;
	my $startLink = idToSlide($slideIds[0]);

	print <<ENDHTML;
<html>
<head>
  <title>$mainTitle</title>
  <link rel=stylesheet href="style.css" type="text/css">
</head>
<body bgcolor="$bgcolor">

$indexHtml

<p>See <a href="$startLink">slides</a> sequentially.
<p>Slide index:
<ul>
ENDHTML

	my $lastHi = idToHiLo($slideIds[0]);
	my $isNewHi = 0;
	foreach my $id (@slideIds) {
		my $num   = idToNum($id);
		my $file  = idToProto($id);
		my $title = extractTitle($file);
		my ($hi, $lo) = idToHiLo($id);
		print "<ul>\n" if $lastHi == $hi && $isNewHi;
		print "</ul>\n" if $lastHi != $hi && !$isNewHi;
		$isNewHi = $lo == 0;
		$lastHi = $hi;
		my $link = idToSlide($id);
		print qq{<li><a href="$link"><b>$num</b> $title</a>\n};
	}
	print "</ul>\n" if $lastHi >= 0 && !$isNewHi;

	print <<ENDHTML;
</ul>
</body>
</html>
ENDHTML
	exit;
}

my $currId = $q->param('i') || $slideIds[0];
$currId =~ s|^/|| if defined $currId;
die "No valid slide id given\n" unless defined $currId && $currId =~ /^\d+$/;
my $currNum = idToNum($currId);
my $currIdx = 0;
foreach (; $currIdx < @slideIds; $currIdx++) {
	last if $slideIds[$currIdx] == $currId;
}
die "No slide id $currId found\n" unless $currIdx < @slideIds;

my $prevId = $currIdx > 0? $slideIds[$currIdx - 1]: undef;
my $nextId = $currIdx < @slideIds - 1? $slideIds[$currIdx + 1]: undef;
my $nextLink = $nextId && idToSlide($nextId);
my $prevLink = $prevId && idToSlide($prevId);
my $nextLinkHtml = defined $nextId? qq(<a href="$nextLink">Next</a>): '[End]';
my $prevLinkHtml = defined $prevId? qq(<a href="$prevLink">Prev</a>): '[Start]';
my $indexLinkHtml = q{<a href="} . idToSlide('index') . qq{">$mainTitle</a>};

my $linkHeaderHtml = "";
$linkHeaderHtml .= qq(\n  <link rel="Next" href="$nextLink">) if $nextLink;
$linkHeaderHtml .= qq(\n  <link rel="Previous" href="$prevLink">) if $prevLink;
$linkHeaderHtml .= qq(\n  <link rel="Home" href="../../">);
$linkHeaderHtml .= qq(\n  <link rel="Author" href="mailto:migo+nospam_at_homemail_com">);
$linkHeaderHtml .= qq(\n  <link rel="Index" href="../">);
$linkHeaderHtml .= qq(\n  <link rel="Contents" href="./">);

my $currFile = idToProto($currId);
my $bodyHtml = `cat '$currFile'`;
$bodyHtml =~ s/^.*<!-- header -->\n//s;
$bodyHtml =~ s/<!-- footer -->\n.*$//s;

my $pageTitle = extractTitle($currFile, $bodyHtml);

print <<ENDH;
<html>
<head>
  <title>$mainTitle - $pageTitle</title>$linkHeaderHtml
  <link rel="stylesheet" href="style.css" type="text/css">
</head>
<body bgcolor="$bgcolor">

<table width="100%" class=status cellpadding=2 cellspacing=0>
<tr>
  <td width=20%>$nextLinkHtml</td>
  <td align=center width=60%><b>$currNum</b> $pageTitle</td>
  <td align=right width=20%>$prevLinkHtml</td>
</tr>
</table>
<!-- header -->

$bodyHtml

<!-- footer -->
<p>
<table width="100%" class=status cellpadding=2 cellspacing=0>
<tr>
  <td width=20%>$nextLinkHtml</td>
  <td align=center width=60%>$indexLinkHtml</td>
  <td align=right width=20%>$prevLinkHtml</td>
</tr>
</table>

</body>
</html>
ENDH
